( * Sensor support for 3-instruction Forth - Assad Ebrahim - 2020-07-25---Sat---21:06 )
( Included: Laser, PIR sensor, 4x4 matrix keypad )

14 constant BUZZER  \ A3

2 constant BUTTON
3 constant LED
13 constant SPEAKER
6  constant LASER

\ 4x4 matrix keypad - 8 pins
\ TX lines, white tabbed, first four counting from left with pins facing up
2  constant padtx1 
3  constant padtx2 
4  constant padtx3 
5  constant padtx4 
\ RX lines, black tabbed, last four counting from the left with pins facing up
6  constant padrx1
7  constant padrx2
8  constant padrx3
9  constant padrx4


\ child friendly - 5 yr old Adam
: h hi ;
: l lo ;

: start-sensors 
	button pullup
	led out
	buzzer out
	speaker out
	led hi
	300 ms
	buzzer hi
	300 ms
	buzzer lo
	led lo
;

\ ========================================================
( * LASER - output * )

: laser-init
	LASER out 
	50 ms
;

: laser-on LASER hi ;
: laser-off LASER lo ;
: laser-flash 10 0 do laser-on 1500 ms laser-off 1500 ms loop ;


\ ========================================================

\ HC-SR501 - PIR Motion Sensor - Seeing Eye (Infrared)
\ LATER: combine with UM-3561 SIREN IC (need 3.3V "on" signal.  Could use a resistor to step down to 3.3V from 5V

: PIR-init 
    12 in  \ PIR output sensor (detect is high) 
    13 out \ to 5V buzzer
;

: read-PIR  cr begin 12 pinread if ." Detect!" cr 13 hi else 13 lo then 0 until ;





\ ========================================================

( * 4x4 Matrix Keypad - Membrane or Button * )
( * How it works: )
(     4 pins are tx/output, 4 are rx/input.  )
(     To figure out if/which key was pressed, uC flips one tx pin live at a time, and polls the rx pins )
(     A pressed button will mean we have row/col and logic determines which key was pressed )

: keypad-init  \ doesn't matter which 4 are input and which 4 are output --- it's symmetric
	padtx1 out padtx2 out padtx3 out padtx4 out
	padrx1 pullup  padrx2 pullup  padrx3 pullup  padrx4 pullup
;

\  !!!!!! ONLY NEEDED DURING DEVELOPMENT OF 4x4 KEYPAD --- with pinread() -- re-use for other polled sensors
\ CONSIDER target routine for write nibble, read nibble
\ CONSIDER use 74165 shift-in register IC to reduce pin count for controlling these keypads from 8 to 3
\ Application: Can map the keys to anything - output or instruction set - e.g. a macrokeypad via Forth...
: poll-keypad
	padtx1 low
	padrx1 pinread 1 - if ." S4 " then  
	padrx2 pinread 1 - if ." S3 " then  
	padrx3 pinread 1 - if ." S2 " then  
	padrx4 pinread 1 - if ." S1 " then
	padtx1 high
	
	padtx2 low
	padrx1 pinread 1 - if ." S8 " then  
	padrx2 pinread 1 - if ." S7 " then  
	padrx3 pinread 1 - if ." S6 " then  
	padrx4 pinread 1 - if ." S5 " then
	padtx2 high
	
	padtx3 low
	padrx1 pinread 1 - if ." S12 " then  
	padrx2 pinread 1 - if ." S11 " then  
	padrx3 pinread 1 - if ." S10 " then  
	padrx4 pinread 1 - if ." S9 "   then
	padtx3 high
	
	padtx4 low
	padrx1 pinread 1 - if ." S16 " then  
	padrx2 pinread 1 - if ." S15 " then  
	padrx3 pinread 1 - if ." S14 " then  
	padrx4 pinread 1 - if ." S13 "  then	
	padtx4 high
;

: keypad-test begin poll-keypad key? until ;

create keypad '0' c, '1' c, '2' c, '3' c, '4' c, '5' c, '6' c, '7' c, '8' c, '9' c, 'A' c, 'B' c, 'C' c, 'D' c, '#' c, '*' c,

: keypadread ( -- val )  \ read matrix membrane keypad and send back 0x0-0xF numbers
	CALL_KEYPAD xcall
	50 ms  \ 50ms works but you get multiple reports per press
	1 read-com if rcv-buffer c@ else $FF then ;

: fastkeypad ( -- key )  \ software polling --- could do other things to interrupt/do other instructions
    begin keypadread dup $FF = if drop 0 else 1 then until ;

: fastkeypad-test ( -- ) 
	 cr begin fastkeypad bytes keypad + c@ emit space key? until ;



\ =================================================

: help-keypad
  cr
  cr  ." KEYPAD (help-keypad)"
  cr  
  cr  ." include 3iForth.fs  3iSensor.fs"
  cr  ." 1 open-com   rx"
  cr
  cr  ." 4X4 MATRIX KEYPAD"
  cr  ." keypad-init"  
  cr  ." create keypad  (16 mappings)"
  cr  ." fastkeypad   fastkeypad-test"
  cr
;

help-keypad
